@@ -668,7 +668,7 @@ static meta *metafix(request_rec *r, const char *buf)
             if (p != NULL) {
                 while (*p) {
                     p += 7;
-                    while (*p && apr_isspace(*p))
+                    while (apr_isspace(*p))
                         ++p;
                     if (*p != '=')
                         continue;
@@ -1142,11 +1142,11 @@ static const char *set_flags(cmd_parms *cmd, void *CFG, const char *arg)
 {
     proxy_html_conf *cfg = CFG;
     if (arg && *arg) {
-        if (!strcmp(arg, "lowercase"))
+        if (!strcasecmp(arg, "lowercase"))
             cfg->flags |= NORM_LC;
-        else if (!strcmp(arg, "dospath"))
+        else if (!strcasecmp(arg, "dospath"))
             cfg->flags |= NORM_MSSLASH;
-        else if (!strcmp(arg, "reset"))
+        else if (!strcasecmp(arg, "reset"))
             cfg->flags |= NORM_RESET;
     }
     return NULL;
@@ -1265,7 +1265,8 @@ static void proxy_html_hooks(apr_pool_t *p)
     ap_hook_pre_config(mod_proxy_html, NULL, NULL, APR_HOOK_MIDDLE);
     ap_hook_insert_filter(proxy_html_insert, NULL, aszSucc, APR_HOOK_MIDDLE);
 }
-module AP_MODULE_DECLARE_DATA proxy_html_module = {
+
+AP_DECLARE_MODULE(proxy_html) = {
     STANDARD20_MODULE_STUFF,
     proxy_html_config,
     proxy_html_merge,
