@@ -354,16 +354,15 @@ static void cgi_child_errfn(apr_pool_t *pool, apr_status_t err,
                             const char *description)
 {
     apr_file_t *stderr_log;
-    char errbuf[200];
 
     apr_file_open_stderr(&stderr_log, pool);
     /* Escape the logged string because it may be something that
      * came in over the network.
      */
     apr_file_printf(stderr_log,
-                    "(%d)%s: %s\n",
+                    "(%d)%pm: %s\n",
                     err,
-                    apr_strerror(err, errbuf, sizeof(errbuf)),
+                    &err,
 #ifndef AP_UNSAFE_ERROR_LOG_UNESCAPED
                     ap_escape_logitem(pool,
 #endif
@@ -436,7 +435,7 @@ static apr_status_t run_cgi_child(apr_file_t **script_out,
         ((rc = apr_procattr_limit_set(procattr, APR_LIMIT_MEM,
                                       conf->limit_mem)) != APR_SUCCESS) ||
 #endif
-#if RLIMIT_NPROC && defined(AP_CGI_USE_RLIMIT)
+#if defined(RLIMIT_NPROC) && defined(AP_CGI_USE_RLIMIT)
         ((rc = apr_procattr_limit_set(procattr, APR_LIMIT_NPROC,
                                       conf->limit_nproc)) != APR_SUCCESS) ||
 #endif
@@ -993,7 +992,7 @@ static int cgi_handler(request_rec *r)
             /* This redirect needs to be a GET no matter what the original
              * method was.
              */
-            r->method = apr_pstrdup(r->pool, "GET");
+            r->method = "GET";
             r->method_number = M_GET;
 
             /* We already read the message body (if any), so don't allow
